# EMA Java Consumer - Posting data to Contribution Channel - Update 2023

- version: 1.0.0
- Download tutorial source code: [Click here to download](https://developers.refinitiv.com/content/dam/devportal/api-families/elektron/elektron-sdk-java/downloads/rcc-contribution-example.zip).
- Last update: Jan 2023
- Compilers: JDK 1.8, JDK 1.11, Open JDK 11
- Prerequisite: Java, Apache Maven, and RCC access credential

## Introduction

In this tutorial we will explain and use the EMA Java API, to contribute data to the Refinitiv Contribution Channel (RCC) directly, without using the Refinitiv Real-Time Distribution System. The reader must have an understanding of a Refinitiv Real-time SDK (EMA) and be familiar with consuming OMM Market Price data. You should also have a basic understanding of EMA Configuration. To understand the contribution concepts, please refer to [this article](https://developers.refinitiv.com/en/article-catalog/article/contributing-your-data-refinitiv).

This tutorial has been updated in 2023 to open the tunnel stream to the RCC server based on the RCC RSSL Login and Directory service status. 

## Contributing Data to Refinitiv Real-Time System

Currently, clients wishing to Contribute data to Refinitiv, typically do so using on-site contribution systems such as MarketLinkIP. To meet the evolving needs of the market, Refinitiv has developed a new Contribution service - the Refinitiv Contribution Channel (RCC). Contributions Channel is provided through a cluster of virtual servers that can provide data optimization & conflation capabilities as well as a much higher rate of content onto Refinitiv Real-Time System.

There are two key mechanisms through which an API developer can contribute data to the Refinitiv head-end. Once contributed, this data can be delivered to other customers who have been permissioned to view it. This service is commonly used by banks and funds to make their data available to the trading community.

The first option is to use and configure the Refinitiv Real-Time Distribution System, to take care of contribution specific connection and login details.

The second option, which is discussed in this tutorial, is for clients who do not have an onsite Refinitiv Real-Time Distribution System installation and will connect to RCC through the Internet. To contribute directly to RCC, an EMA application undertakes the following steps:

1. Initiating Tunnel Stream 
2. Establish an encrypted RSSL connection to the server
3. Register for OMM Login and Directory Domain streams
4. Open a Tunnel stream to the server when the Directory refresh message shows that Contribution Service is Up 
5. Perform a client login via a Tunnel stream
6. Contribute data using post messages

## Tutorial application example files

This tutorial source code consists of three Java files as follows:
- **Contributor.java**: This Java file contains the ```Contributor class``` which is the main application logic class. It receives RCC connection credentials and contribution service information (service name, item name) and interacts with the other two classes to connect and post data.
- **AppClient.java** : This Java file contains the ```AppClient class``` that implements the ```OmmConsumerClient``` interface for handling encrypted RSSL connection events (especially the OMM Login and Directory domain streams) between the API and RCC server.

- **TunnelStreamClient.java** : This Java file contains the ```TunnelStreamClient``` class that manages all tunnel stream connections, events, and posting logic. The source code is based on the EMA sample: *series400\ex440_System_TunStrm* which ships with the SDK distribution, in the Examples directory.

## EMA Java Tutorial - Posting data to Contribution Channel (direct RSSL connection)

Please see more detail on the [EMA Java Consumer - Posting data to Contributions Channel](https://developers.refinitiv.com/en/api-catalog/refinitiv-real-time-opnsrc/rt-sdk-java/tutorials#ema-consumer-posting-data-to-contribution-channel) tutorial page.


## <a id="prerequisite"></a>Tutorial Application Prerequisite
This tutorial application requires the following dependencies software and libraries.
1. Oracle JDK 1.8, 1.11 & 1.17 or OpenJDK 1.8, 1.11 & 1.17.
2. [Apache Maven](https://maven.apache.org/) project management and comprehension tool.
3. Internet connection.  
4. Access to the Refinitiv Contribution Channel (RCC).

Please contact your Refinitiv representative to help you to access the RCC account and services.

## How to run the tutorial application

Firstly, you need to configure the RCC server endpoint and Dictionary files path in the ```EmaConfig.xml``` file.

Open the ```EmaConfig.xml``` file and input the RCC endpoint and Port:

``` XML
<Channel>
    <Name value="Channel_1"/>
    <ChannelType value="ChannelType::RSSL_ENCRYPTED"/>
    ...     
    <Host value="contrib1-....refinitiv.com"/>
    <Port value="443"/>
</Channel>
```

Then, modify the Field Dictionary nodes in the ```EmaConfig.xml``` file to the local ```RDMFieldDictionary``` and ```enumtype.def``` files on your machine.

``` XML
<Dictionary>
    <Name value="Dictionary_2"/>
    <DictionaryType value="DictionaryType::FileDictionary"/>
    <!-- dictionary names are optional: defaulted to RDMFieldDictionary and enumtype.def -->
    <RdmFieldDictionaryFileName value="./RDMFieldDictionary"/>
    <EnumTypeDefFileName value="./enumtype.def"/>
</Dictionary>
```

Now we come to build and compile the project. Open the project folder in the command prompt and then build a project with the following command:

``` Bash
mvn package
```

Once the project compilation is completed, run the following command to start the tutorial application:

``` Bash
java -jar ./target/EMA_Java_RCC_Tutorial-1.0-jar-with-dependencies.jar -username <RCC User> -password <RCC Password> -service <RCC Service Name> -itemName <Post RIC name>
```

Data that is contributed to the production RCC can be viewed by subscribing to that RIC in either desktop applications or any other consumer applications, like the EMA Consumer sample.

## Troubleshooting
### Enable Logging

EMA SDK uses [Java logging package](https://docs.oracle.com/javase/8/docs/api/java/util/logging/package-summary.html) for logging the API messages. The detail, format, and destination of the logger messages can be controlled by configuring the [Java logger properties](https://docs.oracle.com/cd/E57471_01/bigData.100/data_processing_bdd/src/rdp_logging_config.html).

Sample logging.properties file for storing log files in /logs subdirectory:

```
.level=FINEST

handlers=java.util.logging.FileHandler, java.util.logging.ConsoleHandler

java.util.logging.ConsoleHandler.level=FINEST
java.util.logging.ConsoleHandler.formatter=java.util.logging.SimpleFormatter

java.util.logging.FileHandler.level=FINEST
java.util.logging.FileHandler.pattern=./logs/emaj%u.log

# Write 100000 bytes before rotating this file
java.util.logging.FileHandler.limit=50000000

# Number of rotating files to be used
java.util.logging.FileHandler.count=20
java.util.logging.FileHandler.formatter=java.util.logging.SimpleFormatter
# Format timestamp as date/time with millisecond
java.util.logging.SimpleFormatter.format=%1$tY-%1$tm-%1$td %1$tH:%1$tM:%1$tS.%1$tL %4$-7s %2$s %n%5$s
```

EMA Java logging can be enabled, by passing the above properties file to JVM at run time. In addition, it might be helpful to turn on the JDK network logging, to view the security and encryption handshake messages, between the API and the server. Following command line parameters to JVM will enable verbose network and EMA logs.

``` Bash

set LOGGING_CONFIG=-Djava.util.logging.config.file=logs\logging.properties
set NET_DEBUGGING=-Djavax.net.debug=all
java -cp %CLASSPATH% %LOGGING_CONFIG% %NET_DEBUGGING% Contributor
```

Or 

``` Bash
java -Djava.util.logging.config.file=./resources/logging.properties -jar ./target/EMA_Java_RCC_Tutorial-1.0-jar-with-dependencies.jar -username <RCC User> -password <RCC Password> -service <RCC Service Name> -itemName <Post RIC name>
```

## Additional Resources

If you have any further questions I recommend you post them on our Developer
Forum or contact our Data Helpdesk

- [Developer Forum ](https://community.developers.refinitiv.com/index.html?_ga=2.254456904.1515595789.1672197657-1450175381.1646384094)
- [Contact Data Helpdesk](https://my.refinitiv.com/content/mytr/en/helpandsupport.html?_ga=2.254456904.1515595789.1672197657-1450175381.1646384094)

Existing Tutorials mentioned above:
- [EMA Java Tutorials](https://developers.refinitiv.com/en/api-catalog/refinitiv-real-time-opnsrc/rt-sdk-java/tutorials)

## <a id="ref"></a>References

For further details, please check out the following resources:
* [Refinitiv Real-Time SDK Java page](https://developers.refinitiv.com/en/api-catalog/refinitiv-real-time-opnsrc/rt-sdk-java) on the [Refinitiv Developer Community](https://developers.refinitiv.com/) website.
* [Refinitiv Real-Time SDK Family](https://developers.refinitiv.com/en/use-cases-catalog/refinitiv-real-time) page.

For any questions related to this article or the RTSDK page, please use the Developer Community [Q&A Forum](https://community.developers.refinitiv.com/).