///*|----------------------------------------------------------------------------------------------------
// *|            This source code is provided under the Apache 2.0 license      	--
// *|  and is provided AS IS with no warranty or guarantee of fit for purpose.  --
// *|                See the project's LICENSE.md for details.                  					--
// *|           Copyright Refinitiv 2023. All rights reserved.            		--
///*|----------------------------------------------------------------------------------------------------

package com.refinitiv.ema.tutorials.consumer.tutorial6.rcc;

import com.refinitiv.ema.access.*;
import com.refinitiv.ema.domain.login.Login;
import com.refinitiv.ema.rdm.EmaRdm;

/**
 *  This is the main class for RCC Contribution.
 *  It implements OmmConsumerClient to receive RSSL Login and Directory connection events via callbacks.
 */
public class Contributor {

    static String trceUser = "<RCC Username>";
    static String trcePass = "<RCC Password>";
    static String trceServiceName = "DDS_TRCE";
    static String trceItem = "<RCC Posted Item>";

    /**
     * Process Program arguments for getting RCC credentials
     * @param args
     */
    static void readCommandlineArgs(String[] args) {
        try {

            int argsCount = 0;
            while (argsCount < args.length) {
                if (0 == args[argsCount].compareTo("-?")) {
                    return;
                } else if ("-username".equals(args[argsCount])) {
                    trceUser = argsCount < (args.length - 1) ? args[++argsCount] : null;
                    ++argsCount;
                } else if ("-password".equals(args[argsCount])) {
                    trcePass = argsCount < (args.length - 1) ? args[++argsCount] : null;
                    ++argsCount;
                } else if ("-service".equals(args[argsCount])) {
                    trceServiceName = argsCount < (args.length - 1) ? args[++argsCount] : null;
                    ++argsCount;
                } else if ("-itemName".equals(args[argsCount])) {
                    trceItem = argsCount < (args.length - 1) ? args[++argsCount] : null;
                    ++argsCount;
                } else // unrecognized command line argument
                {
                    return;
                }
            }

        } catch (Exception e) {
            System.out.println(e.getMessage());
            return;
        }
    }

    public static void main(String[] args) {

        try {
            System.out.println("Contributing to Refinitiv Contributions Channel");
            AppClient appClient = new AppClient();

            //Parsing Program arguments.
            readCommandlineArgs(args);

            // Create the TunnelStreamClient object to process RCC Tunnel Stream connection
            TunnelStreamClient tunnelClient = new TunnelStreamClient(trceUser, trcePass, trceServiceName, trceItem);

            System.out.println("Starting encrypted connection...");
            // Create an OMM consumer
            OmmConsumer consumer = EmaFactory.createOmmConsumer(EmaFactory.createOmmConsumerConfig(), appClient);
            //.tunnelingKeyStoreFile("KEYSTORE_FILE_NAME")
            //.tunnelingKeyStorePasswd("KEYSTORE_PASSWORD"));

            //Set the OMMConsumer to the TunnelStreamClient so it can use the OMMConsumer to create a tunnel and submit messages
            tunnelClient.setOmmConsumer(consumer);
            appClient.setTunnelStreamClient(tunnelClient);

            //Register to get the source directory message
            //Login.LoginReq loginReq = EmaFactory.Domain.createLoginReq();
            //consumer.registerClient(loginReq.message(), appClient);
            consumer.registerClient(EmaFactory.createReqMsg().domainType(EmaRdm.MMT_DIRECTORY).serviceName(trceServiceName), appClient);

            Thread.sleep(60000);
        } catch (InterruptedException | OmmException excp) {
            System.out.println(excp.getMessage());
        }
    }
}

